﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Routing;

namespace CutOutAPartFromShapeFile
{
    public partial class Form1 : Form
    {
        private ShapeFileFeatureLayer baseLayer;
        private RoutingLayer routeLayer;
        private PointShape currentPositionInWorld;
        private RoutingEngine routingEngine;

        public Form1()
        {
            InitializeComponent();
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            if (String.IsNullOrEmpty(txtNewShapeFileName.Text.Trim()))
            {
                txtNewShapeFileName.Text = Guid.NewGuid().ToString() + ".shp";
            }

            string newFileName = Path.GetDirectoryName(txtFilePath.Text.Trim()) + "\\" + txtNewShapeFileName.Text.Trim();

            ShapeFileFeatureSource.CloneShapeFileStructure(txtFilePath.Text.Trim(), newFileName,OverwriteMode.Overwrite);
            ShapeFileFeatureSource featureSource = new ShapeFileFeatureSource(newFileName, ShapeFileReadWriteMode.ReadWrite);
            featureSource.Open();

            Collection<Feature> featuresInsideBoundingBox = baseLayer.FeatureSource.GetFeaturesInsideBoundingBox(winformsMap1.CurrentExtent, ReturningColumnsType.AllColumns);
            foreach (Feature feature in featuresInsideBoundingBox)
            {
                featureSource.BeginTransaction();
                featureSource.AddFeature(feature);
                featureSource.CommitTransaction();
            }

            MessageBox.Show("The features inside current extent has been saved into the new shape files");
        }

        private void winformsMap1_MapClick(object sender, MapClickWinformsMapEventArgs e)
        {
            if (e.MouseButton == MapMouseButton.Right)
            {
                currentPositionInWorld = e.WorldLocation;
            }
        }

        private void winformsMap1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                mapMenu.Show(winformsMap1, e.Location);
            }
        }

        private void btnBrower_Click(object sender, EventArgs e)
        {
            if (openFiles.ShowDialog() == DialogResult.OK)
            {
                string fileName = openFiles.FileName;
                txtFilePath.Text = fileName;
                // render map
                baseLayer = new ShapeFileFeatureLayer(fileName);
                baseLayer.Open();
                baseLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.LocalRoad4;
                baseLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                winformsMap1.Overlays.Clear();
                LayerOverlay overlay = new LayerOverlay();
                overlay.Layers.Add("baseLayer", baseLayer);
                winformsMap1.Overlays.Add("baseOverlay", overlay);

                LayerOverlay routeOverlay = new LayerOverlay();
                routeLayer = new RoutingLayer();
                routeOverlay.Layers.Add(routeLayer);
                winformsMap1.Overlays.Add("routeOverlay", routeOverlay);

                winformsMap1.CurrentExtent = baseLayer.GetBoundingBox();
                winformsMap1.Refresh();
                // init routingEngine
                RtgRoutingSource routingSource = new RtgRoutingSource(Path.ChangeExtension(fileName, "rtg"));
                routingEngine = new RoutingEngine(routingSource, baseLayer.FeatureSource);
            }
        }

        private void mapMenuItem_Click(object sender, EventArgs e)
        {
            if (routeLayer != null)
            {
                ToolStripMenuItem menuItem = (ToolStripMenuItem)sender;
                routeLayer.Routes.Clear();
                switch (menuItem.Tag.ToString())
                {
                    case "from":
                        routeLayer.StartPoint = currentPositionInWorld;
                        Route();
                        break;
                    case "to":
                        routeLayer.EndPoint = currentPositionInWorld;
                        Route();
                        break;
                    case "clear":
                        routeLayer.StartPoint = null;
                        routeLayer.EndPoint = null;
                        break;
                    default:
                        break;
                }
                LayerOverlay overlay = winformsMap1.Overlays["routeOverlay"] as LayerOverlay;
                winformsMap1.Refresh(overlay);
            }
        }

        private void Route()
        {
            if (routeLayer.StartPoint != null && routeLayer.EndPoint != null && routingEngine != null)
            {
                RoutingResult result = routingEngine.GetRoute(routeLayer.StartPoint, routeLayer.EndPoint);
                routeLayer.Routes.Add(result.Route);
            }
        }
    }
}
